@@ -167,7 +167,7 @@ static guint8 const zero_buf [ZERO_PAD_BUF_SIZE];
 
 /* Calculate the block of the current offset in the file.  A useful idiom is to
  * pad_zero to move to the start of the next block, then get the block number.
- * This avoid fence post type problems with partial blocks. */
+ * This avoids fence post type problems with partial blocks. */
 static inline guint32
 ole_cur_block (GsfOutfileMSOle const *ole)
 {
@@ -177,7 +177,8 @@ ole_cur_block (GsfOutfileMSOle const *ole)
 static inline unsigned
 ole_bytes_left_in_block (GsfOutfileMSOle *ole)
 {
-	unsigned r = (gsf_output_tell (ole->sink) - OLE_HEADER_SIZE) % ole->bb.size;
+	/* blocks are multiples of bb.size (the header is padded out to bb.size) */
+	unsigned r = gsf_output_tell (ole->sink) % ole->bb.size;
 	return (r != 0) ? (ole->bb.size - r) : 0;
 }
 
@@ -248,13 +249,17 @@ gsf_outfile_msole_close_root (GsfOutfileMSOle *ole)
 		GsfOutfileMSOle *child = g_ptr_array_index (elem, i);
 		if (child->type == MSOLE_SMALL_BLOCK) {
 			gsf_off_t size = gsf_output_size (GSF_OUTPUT (child));
-
-			child->blocks = ((size - 1) >> ole->sb.shift) + 1;
-			gsf_output_write (ole->sink, child->blocks << ole->sb.shift,
-				child->content.small_block.buf);
-
-			child->first_block = blocks;
-			blocks += child->blocks;
+			if (size > 0) {
+				child->blocks = ((size - 1) >> ole->sb.shift) + 1;
+				gsf_output_write (ole->sink,
+						  child->blocks << ole->sb.shift,
+						  child->content.small_block.buf);
+				child->first_block = blocks;
+				blocks += child->blocks;
+			} else {
+				child->blocks = 0;
+				child->first_block = BAT_MAGIC_END_OF_CHAIN;
+			}
 		}
 	}
 	data_size = gsf_output_tell (ole->sink) - data_size;
@@ -271,8 +276,8 @@ gsf_outfile_msole_close_root (GsfOutfileMSOle *ole)
 	sbat_start = ole_cur_block (ole);
 	for (i = 0 ; i < elem->len ; i++) {
 		GsfOutfileMSOle *child = g_ptr_array_index (elem, i);
-		if (child->type == MSOLE_SMALL_BLOCK)
-			ole_write_bat (ole->sink,  child->first_block, child->blocks);
+		if (child->type == MSOLE_SMALL_BLOCK && child->blocks > 0)
+			ole_write_bat (ole->sink, child->first_block, child->blocks);
 	}
 	ole_pad_bat_unused (ole, 0);
 	num_sbat = ole_cur_block (ole) - sbat_start;
@@ -310,7 +315,7 @@ gsf_outfile_msole_close_root (GsfOutfileMSOle *ole)
 			memcpy (buf + DIRENT_CLSID, child->clsid, sizeof (child->clsid));
 		} else if (child->type == MSOLE_DIR) {
 			GSF_LE_SET_GUINT8 (buf + DIRENT_TYPE, DIRENT_TYPE_DIR);
-			GSF_LE_SET_GUINT32 (buf + DIRENT_FIRSTBLOCK, DIRENT_MAGIC_END);
+			GSF_LE_SET_GUINT32 (buf + DIRENT_FIRSTBLOCK, BAT_MAGIC_END_OF_CHAIN);
 			GSF_LE_SET_GUINT32 (buf + DIRENT_FILE_SIZE, 0);
 			/* write the class id */
 			memcpy (buf + DIRENT_CLSID, child->clsid, sizeof (child->clsid));
@@ -435,7 +440,7 @@ recalc_bat_bat :
 
 	/* write extended Meta-BAT */
 	if (num_xbat > 0) {
-		gsf_output_seek (ole->sink, (gsf_off_t) 0, G_SEEK_END);
+		gsf_output_seek (ole->sink, 0, G_SEEK_END);
 		for (i = 0 ; i++ < num_xbat ; ) {
 			bat_start += blocks;
 			num_bat   -= blocks;
@@ -473,7 +478,7 @@ gsf_outfile_msole_close (GsfOutput *output)
 		return gsf_outfile_msole_close_root (ole);
 
 	if (ole->type == MSOLE_BIG_BLOCK) {
-		gsf_outfile_msole_seek (output, (gsf_off_t) 0, G_SEEK_END);
+		gsf_outfile_msole_seek (output, 0, G_SEEK_END);
 		ole_pad_zero (ole);
 		ole->blocks = ole_cur_block (ole) - ole->first_block;
 		return gsf_output_unwrap (G_OBJECT (output), ole->sink);
@@ -593,8 +598,9 @@ gsf_outfile_msole_set_block_shift (GsfOutfileMSOle *ole,
 }
 
 static GsfOutput *
-gsf_outfile_msole_new_child (GsfOutfile *parent, char const *name,
-			     gboolean is_dir)
+gsf_outfile_msole_new_child (GsfOutfile *parent,
+			     char const *name, gboolean is_dir,
+			     char const *first_property_name, va_list args)
 {
 	GsfOutfileMSOle *ole_parent = (GsfOutfileMSOle *)parent;
 	GsfOutfileMSOle *child;
@@ -602,7 +608,8 @@ gsf_outfile_msole_new_child (GsfOutfile *parent, char const *name,
 	g_return_val_if_fail (ole_parent != NULL, NULL);
 	g_return_val_if_fail (ole_parent->type == MSOLE_DIR, NULL);
 
-	child = g_object_new (GSF_OUTFILE_MSOLE_TYPE, NULL);
+	child = (GsfOutfileMSOle *)g_object_new_valist (
+		GSF_OUTFILE_MSOLE_TYPE, first_property_name, args);
 	if (is_dir) {
 		child->type = MSOLE_DIR;
 		child->content.dir.children = NULL;
@@ -677,22 +684,21 @@ compute_shift (unsigned value)
 
 /**
  * gsf_outfile_msole_new_full :
- * @sink : a #GsfOutput to hold the OLE2 file
- * @bb_size : size of large blocks
- * @sb_size : size of small blocks
+ * @sink : a #GsfOutput to hold the OLE2 file.
+ * @bb_size : size of large blocks.
+ * @sb_size : size of small blocks.
  *
  * Creates the root directory of an MS OLE file and manages the addition of
  * children.
  *
  * NOTE : adds a reference to @sink
  *
- * Returns : the new ole file handler
+ * Returns the new ole file handler
  **/
 GsfOutfile *
-gsf_outfile_msole_new_full (GsfOutput *sink,
-			    unsigned bb_size, unsigned sb_size)
+gsf_outfile_msole_new_full (GsfOutput *sink, guint bb_size, guint sb_size)
 {
-	static guint8 default_header [] = {
+	static guint8 const default_header [] = {
 /* 0x00 */	0xd0, 0xcf, 0x11, 0xe0, 0xa1, 0xb1, 0x1a, 0xe1,
 /* 0x08 */	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 /* 0x18 */	0x3e, 0x00, 0x03, 0x00, 0xfe, 0xff, 0x09, 0x00,
@@ -718,6 +724,7 @@ gsf_outfile_msole_new_full (GsfOutput *sink,
 	if (ole->bb.size != bb_size ||
 	    ole->sb.size != sb_size ||
 	    bb_size <= sb_size ||
+	    bb_size < DIRENT_SIZE ||
 	    sb_size < 8 ||
 	    ZERO_PAD_BUF_SIZE < ole->bb.size) {
 		if (ZERO_PAD_BUF_SIZE < ole->bb.size)
@@ -745,8 +752,8 @@ gsf_outfile_msole_new_full (GsfOutput *sink,
 	gsf_output_write (sink, OLE_HEADER_SIZE, buf);
 	g_free (buf);
 
-	/* FIXME : Do we need to pad the header out to bb_size for other block
-	 * sizes ? if so do we pad with 0 or something else ? */
+	/* header must be padded out to bb.size with zeros */
+	ole_pad_zero(ole);
 
 	return GSF_OUTFILE (ole);
 }
@@ -760,7 +767,7 @@ gsf_outfile_msole_new_full (GsfOutput *sink,
  *
  * NOTE : adds a reference to @sink
  *
- * Returns : the new ole file handler
+ * Returns the new ole file handler
  **/
 GsfOutfile *
 gsf_outfile_msole_new (GsfOutput *sink)
