@@ -2,10 +2,10 @@
 /*
  * gsf-outfile.h: interface for creating structured files
  *
- * Copyright (C) 2002 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2002-2004 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2.1 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,7 +13,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
@@ -32,10 +32,12 @@ G_BEGIN_DECLS
 
 GType gsf_outfile_get_type (void);
 
-GsfOutput *gsf_outfile_new_child  (GsfOutfile *outfile,
-				   char const *name, gboolean is_dir);
-
-GQuark gsf_outfile_error (void);
+GsfOutput *gsf_outfile_new_child      (GsfOutfile *outfile,
+				       char const *name, gboolean is_dir);
+GsfOutput *gsf_outfile_new_child_full (GsfOutfile *outfile,
+				       char const *name, gboolean is_dir,
+				       char const *first_property_name,
+					   ...);
 
 G_END_DECLS
 
