@@ -2,10 +2,10 @@
 /*
  * gsf-output.h: interface for storing data
  *
- * Copyright (C) 2002 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2002-2004 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2.1 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,7 +13,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
@@ -28,6 +28,34 @@
 
 G_BEGIN_DECLS
 
+typedef struct {
+	GObjectClass g_object_class;
+
+	gboolean (*Close)   (GsfOutput *output);
+	gboolean (*Seek)    (GsfOutput *output,
+			     gsf_off_t offset, GSeekType whence);
+	gboolean (*Write)   (GsfOutput *output,
+			     size_t num_bytes, guint8 const *data);
+	gsf_off_t (*Vprintf) (GsfOutput *output,
+			     char const *format, va_list args);
+} GsfOutputClass;
+#define GSF_OUTPUT_CLASS(k)    (G_TYPE_CHECK_CLASS_CAST ((k), GSF_OUTPUT_TYPE, GsfOutputClass))
+#define GSF_IS_OUTPUT_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GSF_OUTPUT_TYPE))
+
+struct _GsfOutput {
+	GObject g_object;
+
+	/*< protected >*/
+	gsf_off_t   cur_size, cur_offset;
+	char       *name;
+	GObject    *wrapped_by;
+	GsfOutfile *container;
+	GError	   *err;
+	gboolean    is_closed;
+
+	char	   *printf_buf;
+	int	    printf_buf_size;
+};
 #define GSF_OUTPUT_TYPE        (gsf_output_get_type ())
 #define GSF_OUTPUT(o)          (G_TYPE_CHECK_INSTANCE_CAST ((o), GSF_OUTPUT_TYPE, GsfOutput))
 #define GSF_IS_OUTPUT(o)       (G_TYPE_CHECK_INSTANCE_TYPE ((o), GSF_OUTPUT_TYPE))
@@ -36,7 +64,12 @@ GType gsf_output_get_type (void);
 
 char const   *gsf_output_name	   (GsfOutput const *output);
 GsfOutfile   *gsf_output_container (GsfOutput const *output);
+
 GError const *gsf_output_error	   (GsfOutput const *output);
+gboolean gsf_output_set_error	   (GsfOutput *output,
+				    gint        code,
+				    char const *format,
+				    ...) G_GNUC_PRINTF (3, 4);
 
 gsf_off_t     gsf_output_size      (GsfOutput *output);
 gboolean      gsf_output_close     (GsfOutput *output);
@@ -52,9 +85,11 @@ gboolean gsf_output_unwrap (GObject *wrapper, GsfOutput *wrapee);
 
 GQuark gsf_output_error_id (void);
 
-gboolean gsf_output_printf (GsfOutput *output, char const* format,
+gboolean gsf_output_printf (GsfOutput *output, char const *format,
 			    ...) G_GNUC_PRINTF (2, 3);
-gboolean gsf_output_puts (GsfOutput *output, char const* line);
+gsf_off_t gsf_output_vprintf (GsfOutput *output, char const *format,
+			      va_list args);
+gboolean gsf_output_puts (GsfOutput *output, char const *line);
 
 G_END_DECLS
 
