@@ -38,7 +38,7 @@ test (int argc, char *argv[])
 
 	for (i = 1; i < argc; i++) {
 		fprintf (stderr, "%s\n", argv [i]);
-		input = GSF_INPUT (gsf_input_stdio_new (argv[i], &err));
+		input = gsf_input_stdio_new (argv[i], &err);
 		if (input == NULL) {
 
 			g_return_val_if_fail (err != NULL, 1);
@@ -49,7 +49,7 @@ test (int argc, char *argv[])
 		}
 
 		input = gsf_input_uncompress (input);
-		infile = GSF_INFILE (gsf_infile_zip_new (input, &err));
+		infile = gsf_infile_zip_new (input, &err);
 		g_object_unref (G_OBJECT (input));
 
 		if (infile == NULL) {
