@@ -28,21 +28,24 @@
 #include <gsf/gsf-infile.h>
 #include <gsf/gsf-infile-msole.h>
 #include <gsf/gsf-msole-utils.h>
+#include <gsf/gsf-docprop-vector.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
 
 #define BIFF_TYPES_FILE    "biff-types.h"
+/*#define DUMP_CONTENT	1*/
 
 typedef struct {
 	guint16 opcode;
 	char *name;
 } GENERIC_TYPE;
 
+#ifdef DUMP_CONTENT
 static GPtrArray *biff_types   = NULL;
 static void
-read_types (const char *fname, GPtrArray **types)
+read_types (char const *fname, GPtrArray **types)
 {
 	FILE *file = fopen(fname, "r");
 	unsigned char buffer[1024];
@@ -52,7 +55,7 @@ read_types (const char *fname, GPtrArray **types)
 		file = fopen (newname, "r");
 	}
 	if (!file) {
-		printf ("Can't find vital file '%s'\n", fname);
+		g_printerr ("Can't find vital file '%s'\n", fname);
 		return;
 	}
 	while (!feof(file)) {
@@ -90,13 +93,8 @@ get_biff_opcode_name (unsigned opcode)
 	/* Count backwars to give preference to non-filtered record types */
 	for (lp=biff_types->len; --lp >= 0 ;) {
 		GENERIC_TYPE *bt = g_ptr_array_index (biff_types, lp);
-		if (bt->opcode>0xff) {
-			if (bt->opcode == opcode)
-				return bt->name;
-		} else {
-			if (bt->opcode == (opcode&0xff))
-				return bt->name;
-		}
+		if (bt->opcode == opcode)
+			return bt->name;
 	}
 	return "Unknown";
 }
@@ -139,6 +137,37 @@ dump_biff_stream (GsfInput *stream)
 		pos = gsf_input_tell (stream);
 	}
 }
+#endif /* DUMP_CONTENT */
+
+static void
+cb_print_property (char const *name, GsfDocProp const *prop)
+{
+	GValue const *val = gsf_doc_prop_get_val  (prop);
+	char *tmp;
+
+	if (gsf_doc_prop_get_link (prop) != NULL)
+		g_print ("prop '%s' LINKED TO  -> '%s'\n",
+			 name, gsf_doc_prop_get_link (prop));
+	else
+		g_print ("prop '%s'\n", name);
+
+	if (VAL_IS_GSF_DOCPROP_VECTOR ((GValue *)val)) {
+		GValueArray *va = gsf_value_get_docprop_varray (val);
+		unsigned i;
+
+		for (i = 0 ; i < va->n_values; i++) {
+			tmp = g_strdup_value_contents (
+				g_value_array_get_nth (va, i));
+			g_print ("\t[%u] = %s\n", i, tmp);
+			g_free (tmp);
+		}
+	} else {
+		tmp = g_strdup_value_contents (val);
+		printf ("\t= %s\n", tmp);
+		g_free (tmp);
+	}
+}
+
 static int
 test (unsigned argc, char *argv[])
 {
@@ -176,7 +205,9 @@ test (unsigned argc, char *argv[])
 			g_warning ("'%s' Not an OLE file: %s", argv[i], err->message);
 			g_error_free (err);
 
+#ifdef DUMP_CONTENT
 			dump_biff_stream (input);
+#endif
 
 			g_object_unref (G_OBJECT (input));
 			continue;
@@ -195,19 +226,37 @@ test (unsigned argc, char *argv[])
 
 		stream = gsf_infile_child_by_name (infile, "\05SummaryInformation");
 		if (stream != NULL) {
+			GsfDocMetaData *meta_data = gsf_doc_meta_data_new ();
+
 			puts ( "SummaryInfo");
-			gsf_msole_metadata_read (stream, &err);
-			if (err != NULL)
+			err = gsf_msole_metadata_read (stream, meta_data);
+			if (err != NULL) {
 				g_warning ("'%s' error: %s", argv[i], err->message);
+				g_error_free (err);
+				err = NULL;
+			} else
+				gsf_doc_meta_data_foreach (meta_data,
+					(GHFunc) cb_print_property, NULL);
+
+			g_object_unref (meta_data);
 			g_object_unref (G_OBJECT (stream));
 		}
 
 		stream = gsf_infile_child_by_name (infile, "\05DocumentSummaryInformation");
 		if (stream != NULL) {
+			GsfDocMetaData *meta_data = gsf_doc_meta_data_new ();
+
 			puts ( "DocSummaryInfo");
-			gsf_msole_metadata_read (stream, &err);
-			if (err != NULL)
+			err = gsf_msole_metadata_read (stream, meta_data);
+			if (err != NULL) {
 				g_warning ("'%s' error: %s", argv[i], err->message);
+				g_error_free (err);
+				err = NULL;
+			} else
+				gsf_doc_meta_data_foreach (meta_data,
+					(GHFunc) cb_print_property, NULL);
+
+			g_object_unref (meta_data);
 			g_object_unref (G_OBJECT (stream));
 		}
 
@@ -215,7 +264,7 @@ test (unsigned argc, char *argv[])
 			stream = gsf_infile_child_by_name (infile, stream_names[j]);
 			if (stream != NULL) {
 				puts (j < 3 ? "Excel97" : "Excel95");
-				dump_biff_stream (stream);
+/* frank				dump_biff_stream (stream); */
 				g_object_unref (G_OBJECT (stream));
 				break;
 			}
